@@ -187,20 +187,11 @@ static int map_pci_irq_class(unsigned int pci_class)
 static unsigned int read_pci_data(const char *devpath, const char* file)
 {
 	char path[PATH_MAX];
-	FILE *fd;
 	unsigned int data = PCI_INVAL_DATA;
 
 	sprintf(path, "%s/%s", devpath, file);
-
-	fd = fopen(path, "r");
-
-	if (!fd) {
-		log(TO_CONSOLE, LOG_WARNING, "PCI: can't open file:%s\n", path);
-		return data;
-	}
-
-	(void) fscanf(fd, "%x", &data);
-	fclose(fd);
+	if (process_one_line(path, get_hex, &data) < 0)
+		log(TO_CONSOLE, LOG_WARNING, "PCI: can't get from file:%s\n", path);
 
 	return data;
 }
@@ -265,7 +256,7 @@ static gint compare_ints(gconstpointer a, gconstpointer b)
 	return ai->irq - bi->irq;
 }
 
-static void add_banned_irq(int irq, GList **list)
+static void __add_banned_irq(int irq, GList **list)
 {
 	struct irq_info find, *new;
 	GList *entry;
@@ -275,7 +266,7 @@ static void add_banned_irq(int irq, GList **list)
 	if (entry)
 		return;
 
-	new = calloc(sizeof(struct irq_info), 1);
+	new = calloc(1, sizeof(struct irq_info));
 	if (!new) {
 		log(TO_CONSOLE, LOG_WARNING, "No memory to ban irq %d\n", irq);
 		return;
@@ -289,20 +280,14 @@ static void add_banned_irq(int irq, GList **list)
 	return;
 }
 
-void add_cl_banned_irq(int irq)
+void add_banned_irq(int irq)
 {
-	add_banned_irq(irq, &cl_banned_irqs);
+	__add_banned_irq(irq, &banned_irqs);
 }
 
-static int is_banned_irq(int irq)
+void add_cl_banned_irq(int irq)
 {
-	GList *entry;
-	struct irq_info find;
-
-	find.irq = irq;
-
-	entry = g_list_find_custom(banned_irqs, &find, compare_ints);
-	return entry ? 1:0;
+	__add_banned_irq(irq, &cl_banned_irqs);
 }
 
 gint substr_find(gconstpointer a, gconstpointer b)
@@ -343,65 +328,42 @@ void add_cl_banned_module(char *modname)
  * related device. NULL devpath means no sysfs entries for
  * this irq.
  */
-static struct irq_info *add_one_irq_to_db(const char *devpath, int irq, struct user_irq_policy *pol)
+static struct irq_info *add_one_irq_to_db(const char *devpath, struct irq_info *hint, struct user_irq_policy *pol)
 {
-	int irq_class = IRQ_OTHER;
-	struct irq_info *new, find;
+	int irq = hint->irq;
+	struct irq_info *new;
 	int numa_node;
 	char path[PATH_MAX];
-	FILE *fd;
-	char *lcpu_mask;
-	GList *entry;
-	ssize_t ret;
-	size_t blen;
 
-	/*
-	 * First check to make sure this isn't a duplicate entry
-	 */
-	find.irq = irq;
-	entry = g_list_find_custom(interrupts_db, &find, compare_ints);
-	if (entry) {
-		log(TO_CONSOLE, LOG_INFO, "DROPPING DUPLICATE ENTRY FOR IRQ %d on path %s\n", irq, devpath);
-		return NULL;
-	}
-
-	if (is_banned_irq(irq)) {
-		log(TO_ALL, LOG_INFO, "SKIPPING BANNED IRQ %d\n", irq);
-		return NULL;
-	}
-
-	new = calloc(sizeof(struct irq_info), 1);
+	new = calloc(1, sizeof(struct irq_info));
 	if (!new)
 		return NULL;
 
 	new->irq = irq;
-	new->class = IRQ_OTHER;
+	new->type = hint->type;
+	new->class = hint->class;
 
 	interrupts_db = g_list_append(interrupts_db, new);
 
  	/* Some special irqs have NULL devpath */
 	if (devpath != NULL) {
 		/* Map PCI class code to irq class */
-		irq_class = get_irq_class(devpath);
+		int irq_class = get_irq_class(devpath);
 		if (irq_class < 0)
 			goto get_numa_node;
+		new->class = irq_class;
 	}
 
-	new->class = irq_class;
 	if (pol->level >= 0)
 		new->level = pol->level;
 	else
-		new->level = map_class_to_level[irq_class];
+		new->level = map_class_to_level[new->class];
 
 get_numa_node:
-	numa_node = -1;
-	if (numa_avail) {
+	numa_node = NUMA_NO_NODE;
+	if (devpath != NULL && numa_avail) {
 		sprintf(path, "%s/numa_node", devpath);
-		fd = fopen(path, "r");
-		if (fd) {
-			fscanf(fd, "%d", &numa_node);
-			fclose(fd);
-		}
+		process_one_line(path, get_int, &numa_node);
 	}
 
 	if (pol->numa_node_set == 1)
@@ -409,27 +371,33 @@ get_numa_node:
 	else
 		new->numa_node = get_numa_node(numa_node);
 
-	sprintf(path, "%s/local_cpus", devpath);
-	fd = fopen(path, "r");
-	if (!fd) {
-		cpus_setall(new->cpumask);
-		goto out;
+	cpus_setall(new->cpumask);
+	if (devpath != NULL) {
+		sprintf(path, "%s/local_cpus", devpath);
+		process_one_line(path, get_mask_from_bitmap, &new->cpumask);
 	}
-	lcpu_mask = NULL;
-	ret = getline(&lcpu_mask, &blen, fd);
-	fclose(fd);
-	if (ret <= 0) {
-		cpus_setall(new->cpumask);
-	} else {
-		cpumask_parse_user(lcpu_mask, ret, new->cpumask);
-	}
-	free(lcpu_mask);
 
-out:
 	log(TO_CONSOLE, LOG_INFO, "Adding IRQ %d to database\n", irq);
 	return new;
 }
 
+void remove_one_irq_from_db(int irq)
+{
+	struct irq_info find, *tmp;
+	GList *entry = NULL;
+
+	find.irq = irq;
+	entry = g_list_find_custom(interrupts_db, &find, compare_ints);
+	if (!entry)
+		return;
+
+	tmp = entry->data;
+	interrupts_db = g_list_remove(interrupts_db, tmp);
+	free(tmp);
+	log(TO_CONSOLE, LOG_INFO, "IRQ %d was removed from db.\n", irq);
+	return;
+}
+
 static void parse_user_policy_key(char *buf, int irq, struct user_irq_policy *pol)
 {
 	char *key, *value, *end;
@@ -583,6 +551,7 @@ static void get_irq_user_policy(char *path, int irq, struct user_irq_policy *pol
 					break;
 				}
 			}
+			closedir(poldir);
 		}
 	}
 }
@@ -599,7 +568,7 @@ static int check_for_module_ban(char *name)
 		return 0;
 }
 
-static int check_for_irq_ban(char *path __attribute__((unused)), int irq, GList *proc_interrupts)
+static int check_for_irq_ban(int irq, GList *proc_interrupts)
 {
 	struct irq_info find, *res;
 	GList *entry;
@@ -625,6 +594,28 @@ static int check_for_irq_ban(char *path __attribute__((unused)), int irq, GList
 	return 0;
 }
 
+static void add_new_irq(char *path, struct irq_info *hint, GList *proc_interrupts)
+{
+	struct irq_info *new;
+	struct user_irq_policy pol;
+	int irq = hint->irq;
+
+	new = get_irq_info(irq);
+	if (new)
+		return;
+
+	/* Set NULL devpath for the irq has no sysfs entries */
+	get_irq_user_policy(path, irq, &pol);
+	if ((pol.ban == 1) || check_for_irq_ban(irq, proc_interrupts)) { /*FIXME*/
+		__add_banned_irq(irq, &banned_irqs);
+		new = get_irq_info(irq);
+	} else
+		new = add_one_irq_to_db(path, hint, &pol);
+
+	if (!new)
+		log(TO_CONSOLE, LOG_WARNING, "add_new_irq: Failed to add irq %d\n", irq);
+}
+
 /*
  * Figures out which interrupt(s) relate to the device we"re looking at in dirname
  */
@@ -632,15 +623,16 @@ static void build_one_dev_entry(const char *dirname, GList *tmp_irqs)
 {
 	struct dirent *entry;
 	DIR *msidir;
-	FILE *fd;
 	int irqnum;
-	struct irq_info *new;
+	struct irq_info hint;
 	char path[PATH_MAX];
 	char devpath[PATH_MAX];
-	struct user_irq_policy pol;
 
 	sprintf(path, "%s/%s/msi_irqs", SYSPCI_DIR, dirname);
 	sprintf(devpath, "%s/%s", SYSPCI_DIR, dirname);
+
+	/* Needs to be further classified */
+	hint.class = IRQ_OTHER;
 	
 	msidir = opendir(path);
 
@@ -651,18 +643,9 @@ static void build_one_dev_entry(const char *dirname, GList *tmp_irqs)
 				break;
 			irqnum = strtol(entry->d_name, NULL, 10);
 			if (irqnum) {
-				new = get_irq_info(irqnum);
-				if (new)
-					continue;
-				get_irq_user_policy(devpath, irqnum, &pol);
-				if ((pol.ban == 1) || (check_for_irq_ban(devpath, irqnum, tmp_irqs))) {
-					add_banned_irq(irqnum, &banned_irqs);
-					continue;
-				}
-				new = add_one_irq_to_db(devpath, irqnum, &pol);
-				if (!new)
-					continue;
-				new->type = IRQ_TYPE_MSIX;
+				hint.irq = irqnum;
+				hint.type = IRQ_TYPE_MSIX;
+				add_new_irq(devpath, &hint, tmp_irqs);
 			}
 		} while (entry != NULL);
 		closedir(msidir);
@@ -670,10 +653,7 @@ static void build_one_dev_entry(const char *dirname, GList *tmp_irqs)
 	}
 
 	sprintf(path, "%s/%s/irq", SYSPCI_DIR, dirname);
-	fd = fopen(path, "r");
-	if (!fd)
-		return;
-	if (fscanf(fd, "%d", &irqnum) < 0)
+	if (process_one_line(path, get_int, &irqnum) < 0)
 		goto done;
 
 	/*
@@ -685,23 +665,12 @@ static void build_one_dev_entry(const char *dirname, GList *tmp_irqs)
 #else
 	if (irqnum) {
 #endif
-		new = get_irq_info(irqnum);
-		if (new)
-			goto done;
-		get_irq_user_policy(devpath, irqnum, &pol);
-		if ((pol.ban == 1) || (check_for_irq_ban(path, irqnum, tmp_irqs))) {
-			add_banned_irq(irqnum, &banned_irqs);
-			goto done;
-		}
-
-		new = add_one_irq_to_db(devpath, irqnum, &pol);
-		if (!new)
-			goto done;
-		new->type = IRQ_TYPE_LEGACY;
+		hint.irq = irqnum;
+		hint.type = IRQ_TYPE_LEGACY;
+		add_new_irq(devpath, &hint, tmp_irqs);
 	}
 
 done:
-	fclose(fd);
 	return;
 }
 
@@ -726,51 +695,22 @@ void free_cl_opts(void)
 {
 	g_list_free_full(cl_banned_modules, free);
 	g_list_free_full(cl_banned_irqs, free);
-	g_list_free(banned_irqs);
-}
-
-static void add_new_irq(int irq, struct irq_info *hint, GList *proc_interrupts)
-{
-	struct irq_info *new;
-	struct user_irq_policy pol;
-
-	new = get_irq_info(irq);
-	if (new)
-		return;
-
-	/* Set NULL devpath for the irq has no sysfs entries */
-	get_irq_user_policy(NULL, irq, &pol);
-	if ((pol.ban == 1) || check_for_irq_ban(NULL, irq, proc_interrupts)) { /*FIXME*/
-		add_banned_irq(irq, &banned_irqs);
-		new = get_irq_info(irq);
-	} else
-		new = add_one_irq_to_db(NULL, irq, &pol);
-
-	if (!new) {
-		log(TO_CONSOLE, LOG_WARNING, "add_new_irq: Failed to add irq %d\n", irq);
-		return;
-	}
-
-	/*
-	 * Override some of the new irq defaults here
-	 */
-	if (hint) {
-		new->type = hint->type;
-		new->class = hint->class;
-	}
-
-	new->level = map_class_to_level[new->class];
 }
 
 static void add_missing_irq(struct irq_info *info, void *attr)
 {
-	struct irq_info *lookup = get_irq_info(info->irq);
 	GList *proc_interrupts = (GList *) attr;
 
-	if (!lookup)
-		add_new_irq(info->irq, info, proc_interrupts);
+	add_new_irq(NULL, info, proc_interrupts);
 }
 
+static void free_tmp_irqs(gpointer data)
+{
+	struct irq_info *info = data;
+
+	free(info->name);
+	free(info);
+}
 
 void rebuild_irq_db(void)
 {
@@ -800,7 +740,7 @@ void rebuild_irq_db(void)
 
 	for_each_irq(tmp_irqs, add_missing_irq, interrupts_db);
 
-	g_list_free_full(tmp_irqs, free);
+	g_list_free_full(tmp_irqs, free_tmp_irqs);
 
 }
 
