@@ -52,6 +52,7 @@ int foreground_mode;
 int numa_avail;
 int journal_logging = 0;
 int need_rescan;
+int need_rebuild;
 unsigned int log_mask = TO_ALL;
 const char *log_indent;
 unsigned long power_thresh = ULONG_MAX;
@@ -61,9 +62,12 @@ char *pidfile = NULL;
 char *polscript = NULL;
 long HZ;
 int sleep_interval = SLEEP_INTERVAL;
+int last_interval;
 GMainLoop *main_loop;
 
+char *cpu_ban_string = NULL;
 char *banned_cpumask_from_ui = NULL;
+unsigned long migrate_ratio = 0;
 
 static void sleep_approx(int seconds)
 {
@@ -93,6 +97,7 @@ struct option lopts[] = {
 	{"banmod", 1 , NULL, 'm'},
 	{"interval", 1 , NULL, 't'},
 	{"version", 0, NULL, 'V'},
+	{"migrateval", 1, NULL, 'e'},
 	{0, 0, 0, 0}
 };
 
@@ -100,7 +105,7 @@ static void usage(void)
 {
 	log(TO_CONSOLE, LOG_INFO, "irqbalance [--oneshot | -o] [--debug | -d] [--foreground | -f] [--journal | -j]\n");
 	log(TO_CONSOLE, LOG_INFO, "	[--powerthresh= | -p <off> | <n>] [--banirq= | -i <n>] [--banmod= | -m <module>] [--policyscript= | -l <script>]\n");
-	log(TO_CONSOLE, LOG_INFO, "	[--pid= | -s <file>] [--deepestcache= | -c <n>] [--interval= | -t <n>]\n");
+	log(TO_CONSOLE, LOG_INFO, "	[--pid= | -s <file>] [--deepestcache= | -c <n>] [--interval= | -t <n>] [--migrateval= | -e <n>]\n");
 }
 
 static void version(void)
@@ -115,7 +120,7 @@ static void parse_command_line(int argc, char **argv)
 	unsigned long val;
 
 	while ((opt = getopt_long(argc, argv,
-		"odfji:p:s:c:l:m:t:V",
+		"odfjVi:p:s:c:l:m:t:e:",
 		lopts, &longind)) != -1) {
 
 		switch(opt) {
@@ -150,6 +155,7 @@ static void parse_command_line(int argc, char **argv)
 				add_cl_banned_irq((int)val);
 				break;
 			case 'l':
+				free(polscript);
 				polscript = strdup(optarg);
 				break;
 			case 'm':
@@ -183,13 +189,16 @@ static void parse_command_line(int argc, char **argv)
 					exit(1);
 				}
 				break;
+			case 'e':
+				migrate_ratio = strtoul(optarg, NULL, 10);
+				break;
 		}
 	}
 }
 #endif
 
 /*
- * This builds our object tree.  The Heirarchy is typically pretty
+ * This builds our object tree.  The Hierarchy is typically pretty
  * straightforward.
  * At the top are numa_nodes
  * CPU packages belong to a single numa_node, unless the cache domains are in
@@ -251,7 +260,7 @@ gboolean force_rescan(gpointer data __attribute__((unused)))
 	return TRUE;
 }
 
-gboolean scan(gpointer data)
+gboolean scan(gpointer data __attribute__((unused)))
 {
 	log(TO_CONSOLE, LOG_INFO, "\n\n\n-----------------------------------------------------------------------------\n");
 	clear_work_stats();
@@ -259,14 +268,25 @@ gboolean scan(gpointer data)
 
 
 	/* cope with cpu hotplug -- detected during /proc/interrupts parsing */
-	if (need_rescan) {
+	if (need_rescan || need_rebuild) {
+		int try_times = 0;
+
 		need_rescan = 0;
 		cycle_count = 0;
 		log(TO_CONSOLE, LOG_INFO, "Rescanning cpu topology \n");
 		clear_work_stats();
 
-		free_object_tree();
-		build_object_tree();
+		do {
+			free_object_tree();
+			if (++try_times > 3) {
+				log(TO_CONSOLE, LOG_WARNING, "Rescanning cpu topology: fail\n");
+				goto out;
+			}
+
+			need_rebuild = 0;
+			build_object_tree();
+		} while (need_rebuild);
+
 		for_each_irq(NULL, force_rebalance_irq, NULL);
 		parse_proc_interrupts();
 		parse_proc_stat();
@@ -283,15 +303,17 @@ gboolean scan(gpointer data)
 	calculate_placement();
 	activate_mappings();
 
+out:
 	if (debug_mode)
 		dump_tree();
 	if (one_shot_mode)
 		keep_going = 0;
 	cycle_count++;
 
-	if (data != &sleep_interval) {
-		data = &sleep_interval;
-		g_timeout_add_seconds(sleep_interval, scan, data);
+	/* sleep_interval may be changed by socket */
+	if (last_interval != sleep_interval) {
+		last_interval = sleep_interval;
+		g_timeout_add_seconds(sleep_interval, scan, NULL);
 		return FALSE;
 	}
 
@@ -306,10 +328,17 @@ gboolean scan(gpointer data)
 void get_irq_data(struct irq_info *irq, void *data)
 {
 	char **irqdata = (char **)data;
+	char *newptr = NULL;
+
 	if (!*irqdata)
-		*irqdata = calloc(24 + 1 + 11 + 20 + 20 + 11, 1);
+		newptr = calloc(24 + 1 + 11 + 20 + 20 + 11, 1);
 	else
-		*irqdata = realloc(*irqdata, strlen(*irqdata) + 24 + 1 + 11 + 20 + 20 + 11);
+		newptr = realloc(*irqdata, strlen(*irqdata) + 24 + 1 + 11 + 20 + 20 + 11);
+
+	if (!newptr)
+		return;
+
+	*irqdata = newptr;
 
 	sprintf(*irqdata + strlen(*irqdata),
 			"IRQ %d LOAD %lu DIFF %lu CLASS %d ", irq->irq, irq->load,
@@ -320,6 +349,7 @@ void get_object_stat(struct topo_obj *object, void *data)
 {
 	char **stats = (char **)data;	
 	char *irq_data = NULL;
+	char *newptr = NULL;
 	size_t irqdlen;
 
 	if (g_list_length(object->interrupts) > 0) {
@@ -337,11 +367,18 @@ void get_object_stat(struct topo_obj *object, void *data)
 	 * This should be adjusted if the string in the sprintf is changed
 	 */
 	if (!*stats) {
-		*stats = calloc(irqdlen + 31 + 11 + 20 + 11 + 1, 1);
+		newptr = calloc(irqdlen + 31 + 11 + 20 + 11 + 1, 1);
 	} else {
-		*stats = realloc(*stats, strlen(*stats) + irqdlen + 31 + 11 + 20 + 11 + 1);
+		newptr = realloc(*stats, strlen(*stats) + irqdlen + 31 + 11 + 20 + 11 + 1);
 	}
 
+	if (!newptr) {
+		free(irq_data);
+		return;
+	}
+
+	*stats = newptr;
+
 	sprintf(*stats + strlen(*stats), "TYPE %d NUMBER %d LOAD %lu SAVE_MODE %d %s",
 			object->obj_type, object->number, object->load,
 			object->powersave_mode, irq_data ? irq_data : "");
@@ -378,6 +415,10 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 			goto out_close;
 		}
 		cmsg = CMSG_FIRSTHDR(&msg);
+		if (!cmsg) {
+			log(TO_ALL, LOG_WARNING, "Connection no memory.\n");
+			goto out_close;
+		}
 		if ((cmsg->cmsg_level == SOL_SOCKET) &&
 				(cmsg->cmsg_type == SCM_CREDENTIALS)) {
 			struct ucred *credentials = (struct ucred *) CMSG_DATA(cmsg);
@@ -401,6 +442,9 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 							strlen("sleep ")))) {
 				char *sleep_string = malloc(
 						sizeof(char) * (recv_size - strlen("settings sleep ")));
+
+				if (!sleep_string)
+					goto out_close;
 				strncpy(sleep_string, buff + strlen("settings sleep "),
 						recv_size - strlen("settings sleep "));
 				int new_iterval = strtoul(sleep_string, NULL, 10);
@@ -413,6 +457,9 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 				char *end;
 				char *irq_string = malloc(
 						sizeof(char) * (recv_size - strlen("settings ban irqs ")));
+
+				if (!irq_string)
+					goto out_close;
 				strncpy(irq_string, buff + strlen("settings ban irqs "),
 						recv_size - strlen("settings ban irqs "));
 				g_list_free_full(cl_banned_irqs, free);
@@ -429,30 +476,47 @@ gboolean sock_handle(gint fd, GIOCondition condition, gpointer user_data __attri
 				free(irq_string);
 			} else if (!(strncmp(buff + strlen("settings "), "cpus ",
 							strlen("cpus")))) {
-				char *cpu_ban_string = malloc(
+				banned_cpumask_from_ui = NULL;
+				free(cpu_ban_string);
+				cpu_ban_string = NULL;
+
+				cpu_ban_string = malloc(
 						sizeof(char) * (recv_size - strlen("settings cpus ")));
+
+				if (!cpu_ban_string)
+					goto out_close;
 				strncpy(cpu_ban_string, buff + strlen("settings cpus "),
 						recv_size - strlen("settings cpus "));
 				banned_cpumask_from_ui = strtok(cpu_ban_string, " ");
 				if (!strncmp(banned_cpumask_from_ui, "NULL", strlen("NULL"))) {
 					banned_cpumask_from_ui = NULL;
+					free(cpu_ban_string);
+					cpu_ban_string = NULL;
 				}
 				need_rescan = 1;
-				free(cpu_ban_string);
 			}
 		}
 		if (!strncmp(buff, "setup", strlen("setup"))) {
 			char banned[512];
 			char *setup = calloc(strlen("SLEEP  ") + 11 + 1, 1);
+			char *newptr = NULL;
+
+			if (!setup)
+				goto out_close;
 			snprintf(setup, strlen("SLEEP  ") + 11 + 1, "SLEEP %d ", sleep_interval);
 			if(g_list_length(cl_banned_irqs) > 0) {
-				for_each_irq(cl_banned_irqs, get_irq_data, setup);
+				for_each_irq(cl_banned_irqs, get_irq_data, &setup);
 			}
 			cpumask_scnprintf(banned, 512, banned_cpus);
-			setup = realloc(setup, strlen(setup) + strlen(banned) + 7 + 1);
+			newptr = realloc(setup, strlen(setup) + strlen(banned) + 7 + 1);
+			if (!newptr)
+				goto out_free_setup;
+
+			setup = newptr;
 			snprintf(setup + strlen(setup), strlen(banned) + 7 + 1,
 					"BANNED %s", banned);
 			send(sock, setup, strlen(setup), 0);
+out_free_setup:
 			free(setup);
 		}
 
@@ -508,6 +572,7 @@ int init_socket()
 int main(int argc, char** argv)
 {
 	sigset_t sigset, old_sigset;
+	int ret = EXIT_SUCCESS;
 
 	sigemptyset(&sigset);
 	sigaddset(&sigset,SIGINT);
@@ -568,29 +633,15 @@ int main(int argc, char** argv)
 
 	HZ = sysconf(_SC_CLK_TCK);
 	if (HZ == -1) {
-		log(TO_ALL, LOG_WARNING, "Unable to determin HZ defaulting to 100\n");
+		log(TO_ALL, LOG_WARNING, "Unable to determine HZ defaulting to 100\n");
 		HZ = 100;
 	}
-
-	build_object_tree();
-	if (debug_mode)
-		dump_object_tree();
-
-
-	/* On single core UP systems irqbalance obviously has no work to do */
-	if (core_count<2) {
-		char *msg = "Balancing is ineffective on systems with a "
-			    "single cpu.  Shutting down\n";
-
-		log(TO_ALL, LOG_WARNING, "%s", msg);
-		exit(EXIT_SUCCESS);
-	}
-
+	
 	if (!foreground_mode) {
 		int pidfd = -1;
 		if (daemon(0,0))
 			exit(EXIT_FAILURE);
-		/* Write pidfile */
+		/* Write pidfile which can be used to avoid starting multiple instances */
 		if (pidfile && (pidfd = open(pidfile,
 			O_WRONLY | O_CREAT | O_EXCL | O_TRUNC,
 			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) >= 0) {
@@ -601,6 +652,21 @@ int main(int argc, char** argv)
 		}
 	}
 
+	build_object_tree();
+	if (debug_mode)
+		dump_object_tree();
+
+
+	/* On single core UP systems irqbalance obviously has no work to do */
+	if (num_online_cpus() <= 1) {
+		char *msg = "Balancing is ineffective on systems with a "
+			    "single cpu.  Shutting down\n";
+
+		log(TO_ALL, LOG_WARNING, "%s", msg);
+		goto out;
+	}
+
+
 	g_unix_signal_add(SIGINT, handler, NULL);
 	g_unix_signal_add(SIGTERM, handler, NULL);
 	g_unix_signal_add(SIGUSR1, handler, NULL);
@@ -620,25 +686,29 @@ int main(int argc, char** argv)
 	parse_proc_stat();
 
 	if (init_socket()) {
-		return EXIT_FAILURE;
+		ret = EXIT_FAILURE;
+		goto out;
 	}
 	main_loop = g_main_loop_new(NULL, FALSE);
-	int *last_interval = &sleep_interval;
-	g_timeout_add_seconds(sleep_interval, scan, last_interval);
+	last_interval = sleep_interval;
+	g_timeout_add_seconds(sleep_interval, scan, NULL);
 	g_main_loop_run(main_loop);
 
 	g_main_loop_quit(main_loop);
 
+out:
 	free_object_tree();
 	free_cl_opts();
+	free(polscript);
 
 	/* Remove pidfile */
 	if (!foreground_mode && pidfile)
 		unlink(pidfile);
 	/* Remove socket */
-	close(socket_fd);
+	if (socket_fd > 0)
+		close(socket_fd);
 	if (socket_name[0])
 		unlink(socket_name);
 
-	return EXIT_SUCCESS;
+	return ret;
 }
